local btk = require "btk"

bees = {}
beesQueue = {}
beeImage = lg.newImage("beebody.png")
beeQuiet = false

local stingerWidth = 10
local stingerHeight = 10
    
function spawnBee(pos)
    local bee = {
        alive = true
    }

    bee.body = lp.newBody(world, pos.x * normalSize, pos.y * normalSize, "dynamic")
    bee.body:setAngularDamping(0.1)
    bee.shape = lp.newRectangleShape(normalSize / 2, normalSize / 2, normalSize, normalSize)
    bee.fixture = lp.newFixture(bee.body, bee.shape)
    fixtureMap[bee.fixture] = bee
    bee.triggerType = "bee"

    bee.stingerShape = lp.newRectangleShape(normalSize + stingerWidth / 2,
        normalSize / 2,
        stingerWidth, stingerHeight)
    bee.stingerFixture = lp.newFixture(bee.body, bee.stingerShape, 0)
    bee.stingerFixture:setSensor(true)
    fixtureMap[bee.stingerFixture] = bee

    function bee.die()
        bee.diePosition = btk.Vec(bee.body:getPosition())
        bee.dieRotation = bee.body:getAngle()
        bee.body:destroy()
        bee.body = nil
        bee.dieTime = lc.getTime()
        bee.alive = false

        bee.flash()
    end

    function bee.pleaseDie()
        bee.marked = true
    end

    function bee.flash()
        bee.flashTime = lc.getTime()
    end

    bees[#bees + 1] = bee
end

function pleaseSpawnBee(pos)
    beesQueue[#beesQueue + 1] = pos
end

function resetBees()
    beesQueue = {}

    for k, v in ipairs(bees) do
        if v.body then v.body:destroy() end
        bees[k] = nil
    end
end

function updateBees(dt)
    for k, v in ipairs(beesQueue) do
        spawnBee(v)
        beesQueue[k] = nil
    end

    for _, v in ipairs(bees) do
        if v.marked then
            v.die()
            v.marked = nil
        end

        if v.alive then
            local flyPower = 50;
            local target = blob.position

            if beeQuiet then
                if not v.flyChangeTime then v.flyChangeTime = lc.getTime() end
                if not v.flyTarget then v.flyTarget = btk.Vec(lr.random() * 69 * 30, lr.random() * 30 * 30) end
                if not v.flyDuration then v.flyDuration = lr.random() * 8 + 2 end
                
                if lc.getTime() - v.flyChangeTime >= v.flyDuration then
                    v.flyTarget = btk.Vec(lr.random() * 69 * 30, lr.random() * 30 * 30) 
                    v.flyChangeTime = lc.getTime()
                end

                target = v.flyTarget
            end


            v.body:applyForce(0, -9.81 * 30)
            
            local currentDir = v.body:getAngle()
            local trot = 0
            local rotSpeed = math.pi * 2 * 0.5
            local rotDir = 0

            local angleDiff = btk.angleDiff(currentDir, trot)
            if math.abs(angleDiff) < rotSpeed * dt then
                rotDir = 0
                v.body:setAngularVelocity(0)
                v.body:setAngle(0)
            else
                rotDir = btk.rotateDirection(currentDir, trot)
            end

            v.body:applyTorque(rotDir * 300)

            local toBlob = target - btk.Vec(v.body:getPosition())
            v.body:applyForce(unpack(toBlob:norm() * flyPower))

            fixtureMap[v.stingerFixture] = nil
            v.stingerFixture:destroy()
            if toBlob.x > 0 then
                v.direction = "right"

                v.stingerShape = lp.newRectangleShape(-stingerWidth / 2,
                    normalSize / 2,
                    stingerWidth, stingerHeight)
                v.stingerFixture = lp.newFixture(v.body, v.stingerShape, 0)
            else
                v.stingerShape = lp.newRectangleShape(normalSize + stingerWidth / 2,
                    normalSize / 2,
                    stingerWidth, stingerHeight)
                v.stingerFixture = lp.newFixture(v.body, v.stingerShape, 0)
                v.direction = "left"
            end

            v.stingerFixture:setSensor(true)
            fixtureMap[v.stingerFixture] = v
        else
            local p = lc.getTime() - v.dieTime
            if p >= 1 then
                v.position = btk.Vec(v.diePosition.x, 29 * normalSize)
            else
                local p = btk.inQuintic(p)
                v.position = btk.Vec(v.diePosition.x,
                    (1 - p) * v.diePosition.y + 29 * normalSize * p)
            end
        end
    end
end

function drawBees()
    for _, v in ipairs(bees) do
        function setColor(r, g, b)
            if lc.getTime() - (v.flashTime or 0) < 0.4 then
                lg.setColor(r, 0, 0)
            else
                lg.setColor(r, g, b)
            end
        end

        setColor(230, 230, 0)
        
        lg.push()

        if v.alive then
            lg.translate(v.body:getPosition())
            lg.rotate(v.body:getAngle())
        else
            lg.translate(unpack(v.position))
            lg.rotate(v.dieRotation)
        end

        if v.direction == "right" then
            lg.draw(beeImage, 0, 0, 0, -1, 1, 30, 0) 
        else
            lg.draw(beeImage, 0, 0) 
        end

        local wingWidth = normalSize / 2 * 1.2
        local wingHeight = 10
        local p = math.sin((lc.getTime() % 0.6) / 0.6 * math.pi)

        if not v.alive then p = 1 end

        setColor(150, 150, 150)
        lg.rectangle("fill", normalSize / 2 - p * wingWidth, -wingHeight, p * wingWidth, wingHeight)
        setColor(180, 180, 180)
        lg.rectangle("fill", normalSize / 2, -wingHeight, wingWidth * p, wingHeight)

        local stingerHeight = 7
        local stingerDepth = 10
        setColor(70, 70, 70)

        if v.direction == "left" then
            lg.polygon("fill",
                normalSize, normalSize / 2 - stingerHeight / 2,
                normalSize + stingerDepth, normalSize / 2,
                normalSize, normalSize / 2 + stingerHeight / 2)
        else 
            lg.polygon("fill",
                0, normalSize / 2 - stingerHeight / 2,
                -stingerDepth, normalSize / 2,
                0, normalSize / 2 + stingerHeight / 2)
        end

        lg.pop()
    end
end

