local btk = require "btk"

local function Blob(resolution, size, position, duration)
    local self = {}

    self.resolution = resolution
    self.size = size
    self.position = btk.clone(position)
    self.duration = duration

    self.sources = {}
    self.targets = {}

    self.start = 0
    self.isTransitioning = false
    self.progress = 0
    self.currentShape = "compact"

    for i = 1, resolution * 2 do
        self.sources[i] = 0
        self.targets[i] = 0
    end

    self.body = lp.newBody(world, position.x, position.y, "dynamic")
    self.shape = lp.newCircleShape(size)
    self.fixture = lp.newFixture(self.body, self.shape)
    self.fixture:setRestitution(0.2)
    self.fixture:setFriction(0.6)

    self.health = 10

    fixtureMap[self.fixture] = self
    function self:collidedWith(other, myFixture, otherFixture)
        if other.triggerType then
            if other.triggerType == self.currentShape then
                if btk.Vec(self.body:getLinearVelocity()):mag() >= 1000 then
                    other.damage = other.damage + 1
                    if other.damage >= 3 then
                        self.previousSpeed = btk.Vec(self.body:getLinearVelocity())
                        self.deleteTarget = other
                    end
                end
            elseif other.triggerType == "beehive" then
                if btk.Vec(self.body:getLinearVelocity()):mag() > 1000 then
                    pleaseSpawnBee(btk.Vec(40, 30))
                    pleaseSpawnBee(btk.Vec(42, 27))
                    pleaseSpawnBee(btk.Vec(44, 24))
                    pleaseSpawnBee(btk.Vec(46, 27))
                    pleaseSpawnBee(btk.Vec(48, 30))

                    beehiveFlash = lc.getTime()
                    beehiveHealth = beehiveHealth - 1
                end
            elseif other.triggerType == "bee" then
                if otherFixture ~= other.stingerFixture and btk.Vec(self.body:getLinearVelocity()):mag() > 1000 then
                    other.pleaseDie()

                    pleaseSpawnBee(btk.Vec(42, 27))
                    if lr.random() < 0.2 then pleaseSpawnBee(btk.Vec(46, 27)) end
                elseif otherFixture == other.stingerFixture then
                    blob.flash = lc.getTime()
                    blob.health = blob.health - 1
                    other.pleaseDie()
                    pleaseSpawnBee(btk.Vec(42, 27))
                end
            end
        end
    end

    function self:newShape(currentShape, size)
        self.sources = self:getPoints()
        self.currentShape = currentShape

        if size ~= nil then
            self.size = size
        end

        self.isTransitioning = true
        self.start = lc.getTime()
        self.targets = {}
    end

    function self.goCompact()
        self:newShape("compact")

        for i = 1, resolution * 2 do
            self.targets[i] = 0
        end
    end

    function self:goCircle(size)
        self:newShape("circle", size)

        local step = 2 * math.pi / resolution
        for i = 1, resolution do
            local r = ((i - 1) / resolution) * math.pi * 2
            table.insert(self.targets, math.cos(r) * self.size)
            table.insert(self.targets, math.sin(r) * self.size)
        end
    end

    function self:goSquare(size)
        self:newShape("square", size)

        for i = 1, resolution do
            local p = i / resolution

            if p < 0.25 then
                p = p / 0.25
                table.insert(self.targets, -self.size + 2 * p * self.size)
                table.insert(self.targets, -self.size)
            elseif p < 0.5 then
                p = (p - 0.25) / 0.25
                table.insert(self.targets, self.size)
                table.insert(self.targets, -self.size + 2 * p * self.size)
            elseif p < 0.75 then
                p = (p - 0.5) / 0.25
                table.insert(self.targets, self.size - 2 * p * self.size)
                table.insert(self.targets, self.size)
            else
                p = (p - 0.75) / 0.25
                table.insert(self.targets, -self.size)
                table.insert(self.targets, self.size - 2 * p * self.size)
            end
        end
    end

    function self:goTriangle(size)
        self:newShape("triangle", size)

        local height = self.size * 2 / 2 * math.sqrt(3)
        local top = btk.Vec(0, -self.size)
        local right = btk.Vec(self.size, top.y + height)
        local left = btk.Vec(-self.size, top.y + height)

        for i = 1, resolution do
            local p = i / resolution

            if p < 1/3 then
                p = p / (1/3)
                table.insert(self.targets, (1 - p) * top.x + p * right.x)
                table.insert(self.targets, (1 - p) * top.y + p * right.y)
            elseif p < 2/3 then
                p = (p - 1/3) / (1/3)
                table.insert(self.targets, (1 - p) * right.x + p * left.x)
                table.insert(self.targets, (1 - p) * right.y + p * left.y)
            else
                p = (p - 2/3) / (1/3)
                table.insert(self.targets, (1 - p) * right.x + p * left.x)
                table.insert(self.targets, (1 - p) * right.y + p * left.y)
            end
        end
    end

    function self:update(dt)
        if not self.freeze then
            if btk.Vec(self.body:getLinearVelocity()):mag() > 1000 then
                if self.currentShape ~= "triangle" then self:goTriangle() end
            else
                if self.currentShape ~= "circle" then self:goCircle() end
            end
        end

        if self.deleteTarget then
            walls:deleteWall(self.deleteTarget)
            self.body:setLinearVelocity(unpack(self.previousSpeed))

            local bb = self.deleteTarget.bb
            wallParticles:addParticles(bb.x, bb.y, bb.w, bb.h, self.deleteTarget.triggerType, self.previousSpeed:norm())

            self.deleteTarget = nil
        end

        self.position = btk.Vec(self.body:getPosition())

        if not self.isTransitioning then return end
            
        self.progress = lc.getTime() - self.start
        
        if self.progress >= self.duration then
            self.isTransitioning = false
        else
            self.progress = self.progress / self.duration
        end
    end

    function self:getPoints()
        if not self.isTransitioning then
            return self.targets
        end

        local progress = btk.outCubic(self.progress)

        local positions = {}
        for k, v in ipairs(self.sources) do
            positions[k] = (1 - progress) * v + progress * self.targets[k]
        end

        return positions
    end

    function self:draw()
        local positions = self:getPoints()

        lg.push()
        lg.translate(self.position.x, self.position.y)
        lg.rotate(self.body:getAngle())

        lg.setColor(255, 255, 255)
        if lc.getTime() - (blob.flash or 0) < 0.4 then
            lg.setColor(255, 0, 0)
        end
        lg.polygon("fill", positions)

        lg.pop()
    end

    return self
end

return Blob
