local btk = require "btk"

walls = {}
function walls:addWall(x, y, w, h, wallType)
    local wall = {}

    x = x * normalSize;
    y = y * normalSize;
    w = w * normalSize;
    h = h * normalSize;

    wall.bb = btk.AABB(x, y, w, h)
    wall.damage = 0

    wall.body = lp.newBody(world, x, y)
    wall.shape = lp.newRectangleShape(w / 2, h / 2, w, h)
    wall.fixture = lp.newFixture(wall.body, wall.shape)
    wall.triggerType = wallType

    fixtureMap[wall.fixture] = wall
     
    table.insert(self, wall)
end

function walls:deleteWall(wall)
    local index = nil

    for k, v in ipairs(self) do
        if wall == v then index = k end
    end

    if index then
        table.remove(self, index)
        wall.body:destroy()
    end
end

function walls:reset()
    for k, v in ipairs(self) do
        v.body:destroy()
        self[k] = nil
    end
end

function walls:draw()
    lg.setColor(120, 180, 70)
    for _, v in ipairs(self) do
        local p = 1 - v.damage / 3
        lg.setColor(120 * p, 180 * p, 70 * p)
        lg.polygon("fill", v.body:getWorldPoints(v.shape:getPoints()))

        if v.triggerType then
            local coverImg = imgs[v.triggerType]
            lg.setColor(255, 255, 255)
            local quad = lg.newQuad(0, 0, v.bb.w, v.bb.h, normalSize, normalSize)
            lg.draw(coverImg, quad, v.bb.x, v.bb.y)
        end
    end
end

wallParticles = {}
function wallParticles:addParticles(x, y, w, h, wallType, dir)
    for i = x, x + w - 1, normalSize do
        for j = y, y + h - 1, normalSize do
            local wallParticle = {}
            wallParticle.body = lp.newBody(world, i, j, "dynamic")
            wallParticle.shape = lp.newRectangleShape(normalSize / 2, normalSize / 2, normalSize, normalSize)
            wallParticle.fixture = lp.newFixture(wallParticle.body, wallParticle.shape)
            wallParticle.drawType = wallType

            wallParticle.body:setLinearVelocity(unpack(dir * 100))
            
            table.insert(wallParticles, wallParticle)
        end
    end
end

function wallParticles:draw()
    lg.setColor(255, 255, 255)
    for k, v in ipairs(self) do
        local coverImg = imgs[v.drawType]
        local pos = btk.Vec(v.body:getPosition())
        local quad = lg.newQuad(0, 0, normalSize, normalSize, normalSize, normalSize)

        lg.push()

        lg.translate(unpack(pos))
        lg.rotate(v.body:getAngle())
        lg.setColor(120, 180, 70)
        lg.polygon("fill", v.shape:getPoints())
        lg.setColor(255, 255, 255)
        lg.draw(coverImg, quad, 0, 0)

        lg.pop()
    end
end

function wallParticles:reset()
    for k, v in ipairs(self) do
        v.body:destroy()
        self[k] = nil
    end
end
